# encoding: UTF-8
class Split < ActiveRecord::Base
  has_paper_trail
  has_many :amounts, :autosave => true, :dependent => :destroy
  belongs_to :account
  belongs_to :transfer
  validates_presence_of :transfer
 
  delegate :epoch, :popis, :reconciled?, :selected?, :datum, :epoch_id,  :to=> :transfer
#   scope :bal, joins(:amounts).group(:currency).sum(:value)
  scope :expense_or_income, where{(accounts.seq =~ '4%') | (accounts.seq =~ '5%')}
  scope :epoch, where{transfer.epoch_id == '5e74a8f2-7db0-4010-94eb-7fcc3f355c01'}
  
  def amount
    amounts.inject({}) do |sum,a|
      sum[a.currency]= sum[a.currency].to_f + a.value
      sum
    end
  end
  def amount_json
    amount.to_json
  end
  def amount=(new)
    old = amount
    (old.keys-new.keys).each{|key|
      self.amounts.where{currency==key}.each{|a| a.destroy}
    }
    (new.keys&old.keys).each{|key|
      am =self.amounts.where{currency==key}.first
      am.value=new[key]
      am.save
    }
    (new.keys-old.keys).each{|key|
      self.amounts << Amount.new(:currency=>key, :value=>new[key])
    }
  end
  # Sečte amounts od všech poslaných splitů
  # @param  splits [Split ActiveRecord::Relation] očekává Relation určující množinu splitů které chceme sečíst
  # @example příklad použití 
  #  @balance = Split.sum_amount(@splits.limit(@show_splits.offset))
  def self.sum_amount(splits_relation)
    Split.from(Arel.sql("(#{splits_relation.to_sql}) as splits")).
      joins{amounts}.group{amounts.currency}.sum('amounts.value')
  end
  def self.balance(options={})
    raise 'Usage of options currency and convert_to at the same time has undefined result' if options[:currency] and options[:convert_to]
    
    if options[:accounts]
      accounts = options.delete(:accounts).split(',')
      return accounts.collect{ |account| Split.balance(options.merge({:account=>account}))}.sum
    else
      result = Split.all_with(options).joins{amounts}.group{amounts.currency}.sum(:value) || {'CZK' => 0}
      if options[:currency]
        return result[options[:currency]] || 0
      elsif options[:convert_to]
        return result.to_multicash_h.convert_to(options[:convert_to])
      else
        return result
      end
    end
  end

  # all_with(options) returns splits which satisfy some options (returns scope)
  #   :account => <account> or <acct_id> vrací jen splity z tohoto Accountu
  #   :only_proper_splits => true   ignoruje splity ze (sub)accountů
  #   :expense_or_income => true    vrací jen splity z (sub)accountů co jsou výdaje či příjmy
  #   :epoch => <epoch>             vrací jen splity patřící do <epoch>
  #   :before_epoch => <epoch>      vrací jen splity patřící do epoch které předcházejí epoše <epoch> (:before_epoch wins to :epoch)
  #   :inclusive_before => true     ovlivňuje :before_epoch a povoluje i splity s epochou ep
  #   :to_date => <datum>           počítá balance jen do datumu <datum> včetně
  #   :to_date2 => <datum>          počítá balance jen do datumu2 <datum> včetně
  #   :seq=> <seq>                  vyhledá jen splity jejichž transfer má tyto seq TODO: works only without epoch options
  #
  # vrácený scope má najoinováno accounts
  def self.all_with(options={})
    if options[:account]
      account = options[:account].is_a?(Account) ? options[:account] : Account.find(options[:account].to_s) 
      raise 'Invalid option[:account] type of acount not found' unless account.is_a?(Account) 
      
      if options[:only_proper_splits]
        pom = account.proper_splits
      else
        pom = account.splits
      end
    else
      pom = Split.scoped
    end
    if options[:epoch].nil? and options[:before_epoch].nil?
      if options[:expense_or_income]
        pom = pom.where{(accounts.seq =~ '4%') | (accounts.seq =~ '5%')}
      end
      if options[:seq]
        pom = pom.joins{transfer}.where{(transfer.seq == options[:seq])}
      end
    else
      pom = pom.joins{transfer}
      if options[:before_epoch].nil?
        epoch_id = options[:epoch].is_a?(Epoch) ? options[:epoch].id : options[:epoch]
        if options[:expense_or_income]
          pom = pom.where{|s| (s.transfer.epoch_id == epoch_id) & ((s.accounts.seq =~ '4%') | (s.accounts.seq =~ '5%'))}
        else
          pom = pom.where{|s| s.transfer.epoch_id == epoch_id}
        end
      else
        if options[:expense_or_income]
          if options[:inclusive_before].nil?
            pom = pom.joins{transfer.epoch}.where{|s| (s.transfer.epoch.starts_at < options[:before_epoch][:starts_at]) & ((s.accounts.seq =~ '4%') | (s.accounts.seq =~ '5%'))}
          else
            pom = pom.joins{transfer.epoch}.where{|s| (s.transfer.epoch.starts_at <= options[:before_epoch][:starts_at]) & ((s.accounts.seq =~ '4%') | (s.accounts.seq =~ '5%'))}          
          end
        else
          if options[:inclusive_before].nil?
            pom = pom.joins{transfer.epoch}.where{|s| s.transfer.epoch.starts_at < options[:before_epoch][:starts_at]}
          else
            pom = pom.joins{transfer.epoch}.where{|s| s.transfer.epoch.starts_at <= options[:before_epoch][:starts_at]}          
          end
        end      
      end
    end
    pom
  end

end
